
//-------------------------------------------------------------

#include <stdio.h>
#include <string.h>

#include "tabs.h" // tables (used for various things)

//-------------------------------------------------------------

// Block size is how large of a chunk we will work with of the input file at a time
#define BLOCK_SIZE	0x100000

FILE *fp, *fz, *fk;	// File in, out, and key file
unsigned char *src;	// source
unsigned char *key;	// key
unsigned int nLen;	// source length
unsigned int kLen;	// key length

unsigned char xor1;	// xor1
unsigned char *tab_a;	// pointer to tab_00 or tab_02
unsigned char *tab_b;	// pointer to tab_01 or tab_03

//-------------------------------------------------------------

// xor m & n by 1 byte from table 'a' and table 'b' respectively.
// set which bit is used in a horribly convoluded way.
#define _xor1_0(j, k) m ^= tab_a[tab_b[m ^ key[(((i + m) ^ j) & xor1) % kLen]] ^ n] & (0x80 >> k); \
		      n ^= tab_b[tab_a[n ^ key[(((i + n) ^ j) & xor1) % kLen]] ^ m] & (0x01 << k); \

#define _xor1(j) _xor1_0(j ^ (xor1 & 7), j);

// xor by 8 bits of i (the address) set by tables
#define ixor(tab0, tab1) xor1 ^= i >> (((tab0[i & n] >> 4) & 5) | (tab1[m] & 2)); \

// rotate xor1's bits around a bit
#define rotate(j) xor1 = (xor1 << j) | (xor1 >> (8-j)); \

//-------------------------------------------------------------

unsigned char byte_xor(unsigned int i)
{
	unsigned char m, n;

	m = i >> 0x18;						// seed m
	n = i >> 0x10;						// seed n

	_xor1(0)						// xor m, n
	_xor1(1)
	_xor1(2)
	_xor1(3)
	_xor1(4)
	_xor1(5)
	_xor1(6)
	_xor1(7)

	tab_a = key[n % kLen] & 0x01 ? tab_00 : tab_02;		// set tables again, this way the data doesn't
	tab_b = key[m % kLen] & 0x02 ? tab_01 : tab_03; 	// have to be xored by the same table as m & n

	rotate((m & 7))						// rotate

	ixor(tab_a, tab_b)					// xor using address
	ixor(tab_b, tab_a)

	rotate((n & 7))						// rotate

	xor1 ^= tab_a[xor1 ^ key[xor1 % kLen]];			// xor using table 'a'
	xor1 ^= tab_b[i % ((m + n)+1)];				// xor using table 'b'

	return xor1;
}

//-------------------------------------------------------------

void crypt(unsigned char *src, unsigned int nLen)
{
	unsigned int i, j;
	unsigned int block = 0;

	tab_a = key[0] & 0x01 ? tab_00 : tab_02;	// seed tables
	tab_b = key[1] & 0x02 ? tab_01 : tab_03;

	xor1 = (key[0] << 4) | (key[1] & 0x0f);		// seed xor

	for (i = 0; i < nLen; i+= BLOCK_SIZE)
	{
		block = (nLen - i) < BLOCK_SIZE ? (nLen - i) : BLOCK_SIZE;	// set block size (Max or what's left)

		fread (src, 1, block, fp);					// read block

		for (j = 0; j < block; j++)					// apply encryption to every byte
		{
			src[j] ^= byte_xor(i + j);				// xor1
		}

		fwrite (src, 1, block, fz);					// write block
	}
}


//-------------------------------------------------------------

void rep_ext(char *filein)		// replace old file extension with .enc
{
	int i;
	for (i = strlen(filein) - 1; i > 0; i--) {
		if (filein[i] == '.') {
			strncpy (filein + i + 1, "enc\0", 5);
			return;
		}
	}
}

int main(int argc, char **argv)
{
	char fileout[256];

	char tkey[128];

	if (argc < 2)
	{
		char filein[256];
		printf ("Please type the name of the file to be en/de-crypted\n");
		gets (filein);
		fp = fopen(filein, "rb");

		memcpy (fileout, filein, strlen(filein));
	}
	else
	{
		fp = fopen(argv[1], "rb");
		memcpy (fileout, argv[1], strlen(argv[1]));
	}

	rep_ext(fileout);

	if (fp == NULL)
	{
		printf ("File not found\n");
		return 1;
	}

	fseek (fp, 0, SEEK_END);
	nLen = ftell (fp);
	rewind (fp);

	if (argc < 3)
	{
		printf ("Would you like to use a key file? (Y/N)\n");
		gets (tkey);

		if (tkey[0] == 'y' || tkey[0] == 'Y')
		{
			char filein[256];
			printf ("Please type the name of your key file\n");
			gets (filein);

			fk = fopen(filein, "rb");

			if (fk == NULL)
			{
				printf ("File not found\n");
				fclose (fp);
				return 1;
			}

			fseek (fk, 0, SEEK_END);
			kLen = ftell (fk);
			rewind (fk);

			key = (unsigned char*)malloc( kLen );

			fread (key, 1, kLen, fk);
			fclose (fk);
		}
		else
		{
			printf ("Please type the password\n");
			gets (tkey);
			key = tkey + 0;
			kLen = strlen(tkey);
		}
	}
	else
	{
		key = argv[2];
		kLen = strlen(argv[2]);
	}

	fz = fopen(fileout, "wb");

	src = (unsigned char*)malloc( BLOCK_SIZE );

	crypt(src, nLen);

	fclose (fp);
	fclose (fz);

	free (key);
	free (src);
}
